import { BtModuleKeys } from "../safeLoadBtModule/types";
import { loadScript } from "../script-utils";

declare global {
  interface Window {
    braintree: Partial<
      Record<BtModuleKeys, { create: () => void; VERSION: string }>
    > & {
      fastlane?: unknown;
      loadFastlane?: ({
        unminified,
      }: {
        unminified?: boolean;
      }) => Promise<unknown>;
    };
  }
}

describe("loadScript", () => {
  let createElementSpy: jest.SpyInstance;
  let appendChildSpy: jest.SpyInstance;
  let scriptMock: Partial<HTMLScriptElement>;
  let eventListeners: { [key: string]: EventListener };

  beforeEach(() => {
    eventListeners = {};
    scriptMock = {
      addEventListener: jest.fn((event: string, callback: EventListener) => {
        eventListeners[event] = callback;
      }),
      set src(value: string) {
        // Simulate script load
        if (value === "valid-src") {
          setTimeout(() => {
            eventListeners["load"]?.(new Event("load"));
          }, 0);
        } else {
          setTimeout(() => {
            eventListeners["error"]?.(new Event("error"));
          }, 0);
        }
      },
    };

    createElementSpy = jest
      .spyOn(document, "createElement")
      .mockReturnValue(scriptMock as HTMLScriptElement);
    appendChildSpy = jest
      .spyOn(document.head, "appendChild")
      .mockImplementation(jest.fn());
  });

  afterEach(() => {
    jest.restoreAllMocks();
  });

  it("should resolve with the exported symbol when script loads successfully", async () => {
    const mockFastLane = { create: jest.fn };
    const mockLoadFastlane = jest.fn().mockResolvedValue(mockFastLane);
    window.braintree = window.braintree || {};
    window.braintree.loadFastlane = mockLoadFastlane;

    const result = await loadScript({
      src: "valid-src",
      name: "Test Script",
      exportedName: "braintree.loadFastlane",
    });

    expect(result).toBe(mockLoadFastlane);
    expect(createElementSpy).toHaveBeenCalledWith("script");
    expect(appendChildSpy).toHaveBeenCalled();
  });

  it("should resolve with undefined if no exportedName is provided", async () => {
    const result = await loadScript({
      src: "valid-src",
      name: "Test Script",
    });

    expect(result).toBeUndefined();
  });

  it("should reject if the script fails to load", async () => {
    await expect(
      loadScript({
        src: "invalid-src",
        name: "Test Script",
      })
    ).rejects.toThrow("Failed to load Test Script @ invalid-src.");
  });

  it("should reject if the script load is aborted", async () => {
    scriptMock.addEventListener = jest.fn((event, callback) => {
      if (event === "abort") {
        setTimeout(() => {
          if (typeof callback === "function") {
            callback(new Event("abort"));
          }
        }, 0);
      }
    });

    await expect(
      loadScript({
        src: "valid-src",
        name: "Test Script",
      })
    ).rejects.toThrow("Test Script @ valid-src load was aborted.");
  });
});
